@@ -18,120 +18,6 @@
 extern "C" {
 #endif
 
-#ifndef FCS_DBM_WITHOUT_CACHES
-
-static int fc_solve_compare_pre_cache_keys(
-    const void *const void_a, const void *const void_b, void *const context)
-{
-#define GET_PARAM(p) ((((const pre_cache_key_val_pair *)(p))->key))
-    return memcmp(
-        &(GET_PARAM(void_a)), &(GET_PARAM(void_b)), sizeof(GET_PARAM(void_a)));
-#undef GET_PARAM
-}
-
-static inline void pre_cache_init(
-    fcs_pre_cache *const pre_cache_ptr, meta_allocator *const meta_alloc)
-{
-    pre_cache_ptr->tree_recycle_bin = NULL;
-    pre_cache_ptr->kaz_tree =
-        fc_solve_kaz_tree_create(fc_solve_compare_pre_cache_keys, NULL,
-            meta_alloc, &(pre_cache_ptr->tree_recycle_bin));
-
-    fc_solve_compact_allocator_init(&(pre_cache_ptr->kv_allocator), meta_alloc);
-    pre_cache_ptr->kv_recycle_bin = NULL;
-    pre_cache_ptr->count_elements = 0;
-}
-
-static inline void pre_cache_insert(fcs_pre_cache *pre_cache,
-    fcs_encoded_state_buffer *key, fcs_encoded_state_buffer *parent)
-{
-    pre_cache_key_val_pair *to_insert;
-
-    if (pre_cache->kv_recycle_bin)
-    {
-        pre_cache->kv_recycle_bin =
-            (to_insert = pre_cache->kv_recycle_bin)->next;
-    }
-    else
-    {
-        to_insert = fcs_compact_alloc_ptr(
-            &(pre_cache->kv_allocator), sizeof(*to_insert));
-    }
-    to_insert->key = *key;
-    to_insert->parent = *parent;
-
-    fc_solve_kaz_tree_alloc_insert(pre_cache->kaz_tree, to_insert);
-    ++pre_cache->count_elements;
-}
-
-static inline bool pre_cache_does_key_exist(
-    fcs_pre_cache *pre_cache, fcs_encoded_state_buffer *key)
-{
-    pre_cache_key_val_pair to_check = {.key = *key};
-    return (
-        fc_solve_kaz_tree_lookup_value(pre_cache->kaz_tree, &to_check) != NULL);
-}
-
-static inline void cache_populate_from_pre_cache(
-    fcs_lru_cache *const cache, fcs_pre_cache *const pre_cache)
-{
-#ifdef FCS_DBM_USE_LIBAVL
-    struct rb_traverser trav;
-    var_AUTO(kaz_tree, pre_cache->kaz_tree);
-    rb_t_init(&trav, kaz_tree);
-
-    for (dict_key_t item = rb_t_first(&trav, kaz_tree); item;
-         item = rb_t_next(&trav))
-    {
-        cache_insert(
-            cache, &(((pre_cache_key_val_pair *)(item))->key), NULL, '\0');
-    }
-#else
-    var_AUTO(kaz_tree, pre_cache->kaz_tree);
-    for (dnode_t *node = fc_solve_kaz_tree_first(kaz_tree); node;
-         node = fc_solve_kaz_tree_next(kaz_tree, node))
-    {
-        cache_insert(
-            cache, &(((pre_cache_key_val_pair *)(node->dict_key))->key));
-    }
-#endif
-}
-
-static inline void pre_cache_offload_and_destroy(fcs_pre_cache *const pre_cache,
-    fcs_dbm_store store, fcs_lru_cache *const cache)
-{
-    fc_solve_dbm_store_offload_pre_cache(store, pre_cache);
-    cache_populate_from_pre_cache(cache, pre_cache);
-
-    // Now reset the pre_cache.
-    fc_solve_kaz_tree_destroy(pre_cache->kaz_tree);
-    fc_solve_compact_allocator_finish(&(pre_cache->kv_allocator));
-}
-
-#ifndef FCS_DBM_CACHE_ONLY
-#define PRE_CACHE_OFFLOAD(instance)                                            \
-    pre_cache_offload_and_destroy(&((instance)->cache_store.pre_cache),        \
-        (instance)->cache_store.store, &((instance)->cache_store.cache))
-#else
-#define PRE_CACHE_OFFLOAD(instance)                                            \
-    {                                                                          \
-    }
-#endif
-
-#ifndef FCS_DBM_CACHE_ONLY
-
-static inline void pre_cache_offload_and_reset(fcs_pre_cache *const pre_cache,
-    const fcs_dbm_store store, fcs_lru_cache *const cache,
-    meta_allocator *const meta_alloc)
-{
-    pre_cache_offload_and_destroy(pre_cache, store, cache);
-    pre_cache_init(pre_cache, meta_alloc);
-}
-
-#endif
-
-#endif // FCS_DBM_WITHOUT_CACHES
-
 typedef struct fcs_dbm_solver_thread_struct dbm_solver_thread;
 
 typedef struct
@@ -139,27 +25,17 @@ typedef struct
     dbm_solver_thread *thread;
 } thread_arg;
 
-static inline void instance_check_key(
-    dbm_solver_thread *const thread, dbm_solver_instance *const instance,
-    const size_t key_depth, fcs_encoded_state_buffer *const key,
-    fcs_dbm_store_val parent, const uint8_t move,
-    const fcs_which_moves_bitmask *const which_irreversible_moves_bitmask
-#ifndef FCS_DBM_WITHOUT_CACHES
-    ,
-    const fcs_fcc_move *moves_to_parent
-#endif
-);
+static inline void instance_check_key(dbm_solver_thread *const thread,
+    dbm_solver_instance *const instance, const size_t key_depth,
+    fcs_encoded_state_buffer *const key,
+    const fcs_encoded_state_buffer raw_parent, const uint8_t move,
+    const fcs_which_moves_bitmask *const which_irreversible_moves_bitmask);
 
-static inline bool instance_check_multiple_keys(
-    dbm_solver_thread *const thread, dbm_solver_instance *const instance,
+static inline bool instance_check_multiple_keys(dbm_solver_thread *const thread,
+    dbm_solver_instance *const instance,
     fcs_dbm__cache_store__common *const cache_store GCC_UNUSED,
     meta_allocator *const meta_alloc GCC_UNUSED, fcs_derived_state **lists,
-    size_t batch_size
-#ifndef FCS_DBM_WITHOUT_CACHES
-    ,
-    const fcs_fcc_move *moves_to_parent
-#endif
-)
+    size_t batch_size)
 {
     // Small optimization in case the list is empty.
     if (batch_size == 1 && !lists[0])
@@ -172,26 +48,11 @@ static inline bool instance_check_multiple_keys(
         for (var_AUTO(list, lists[batch_i]); list; list = list->next)
         {
             instance_check_key(thread, instance, CHECK_KEY_CALC_DEPTH(),
-                &(list->key), list->parent, list->move,
-                &(list->which_irreversible_moves_bitmask)
-#ifndef FCS_DBM_WITHOUT_CACHES
-                    ,
-                moves_to_parent
-#endif
-            );
-#ifndef FCS_DBM_WITHOUT_CACHES
-#ifndef FCS_DBM_CACHE_ONLY
-            if (cache_store->pre_cache.count_elements >=
-                cache_store->pre_cache_max_count)
-            {
-                pre_cache_offload_and_reset(&(cache_store->pre_cache),
-                    cache_store->store, &(cache_store->cache), meta_alloc);
-            }
-#endif
-#endif
+                &(list->key_and_parent.key), list->key_and_parent.parent,
+                list->move, &(list->which_irreversible_moves_bitmask));
         }
     }
-#ifdef MAX_FCC_DEPTH
+#ifdef IN_DEPTH_DBM_SOLVER_dot_c
     const bool have_more = !q_stats_is_empty(
         &instance->colls_by_depth[instance->curr_depth].queue.stats);
 
@@ -251,8 +112,32 @@ static inline void instance_debug_out_state(
     }
 #endif
 
-#ifndef FCS_DBM__VAL_IS_ANCESTOR
-static void calc_trace(fcs_dbm_record *const ptr_initial_record,
+static bool dbm_lookup_parent(const size_t count_stores, fcs_dbm_store *stores,
+    const fcs_encoded_state_buffer key, fcs_encoded_state_buffer *const parent)
+{
+#ifdef FCS_DBM__STORE_KEYS_ONLY
+#include "dbm_parent_lookup.h"
+    var_AUTO(store, stores[0]);
+    var_AUTO(parent_lookup_ptr, fc_solve_dbm_store__get_parent_lookup(store));
+    const_AUTO(ret,
+        rawdump_parent_lookup__lookup_parent(parent_lookup_ptr, key, parent));
+    return ret;
+#else
+    for (size_t i = 0; i < count_stores; ++i)
+    {
+        var_AUTO(store, stores[i]);
+        if (fc_solve_dbm_store_lookup_parent(store, key.s, parent->s))
+
+        {
+            return true;
+        }
+    }
+    return false;
+#endif
+}
+
+static void calc_trace(const size_t count_stores, fcs_dbm_store *stores,
+    fcs_dbm_record *const ptr_initial_record,
     fcs_encoded_state_buffer **const ptr_trace, size_t *const ptr_trace_num)
 {
 #define GROW_BY 100
@@ -260,23 +145,33 @@ static void calc_trace(fcs_dbm_record *const ptr_initial_record,
     size_t trace_max_num = GROW_BY;
     fcs_encoded_state_buffer *trace = SMALLOC(trace, trace_max_num);
     fcs_encoded_state_buffer *key_ptr = trace;
-    fcs_dbm_record *record = ptr_initial_record;
+    fcs_encoded_state_buffer parent;
+    fcs_dbm_record init_record = *ptr_initial_record;
+    fcs_dbm_record *record = &init_record;
 
     while (record)
     {
-#if 1
         *(key_ptr) = record->key;
         if ((++trace_num) == trace_max_num)
         {
             trace = SREALLOC(trace, trace_max_num += GROW_BY);
             key_ptr = &(trace[trace_num - 1]);
         }
-        record = fcs_dbm_record_get_parent_ptr(record);
+        const_AUTO(
+            found, dbm_lookup_parent(count_stores, stores, *key_ptr, &parent));
+        if (found)
+        {
+            record->parent = record->key;
+            record->key = parent;
+        }
+        else
+        {
+            record = NULL;
+        }
         key_ptr++;
-#endif
     }
 #undef GROW_BY
-    *ptr_trace_num = trace_num;
+    *ptr_trace_num = trace_num - 1;
     *ptr_trace = trace;
 }
 
@@ -284,7 +179,11 @@ static inline void mark_and_sweep_old_states(
     dbm_solver_instance *const instance GCC_UNUSED,
     dict_t *const kaz_tree GCC_UNUSED, const size_t curr_depth GCC_UNUSED)
 {
-#ifndef FCS_NO_DBM_AVL
+#ifdef FCS_NO_DBM_AVL
+#error FCS_NO_DBM_AVL
+#else
+#if defined(FCS_DBM__STORE_KEYS_ONLY)
+
     // Now that we are about to descend to a new depth, let's
     // mark-and-sweep the old states, some of which are no longer of interest.
     FILE *const out_fh = instance->common.out_fh;
@@ -299,12 +198,17 @@ static inline void mark_and_sweep_old_states(
     const size_t items_count = kaz_tree->rb_count;
     size_t idx = 0;
     for (dict_key_t item = rb_t_first(&trav, kaz_tree); item;
-         item = rb_t_next(&trav))
+        item = rb_t_next(&trav))
     {
+#if 0
         if (!rb_get_decommissioned_flag(item))
         {
             var_AUTO(ancestor, (struct rb_node *)item);
+#if 0
             while (fcs_dbm_record_get_refcount(&(ancestor->rb_data)) == 0)
+#else
+            while (true)
+#endif
             {
                 rb_set_decommissioned_flag(ancestor, 1);
 
@@ -320,7 +224,11 @@ static inline void mark_and_sweep_old_states(
                 fcs_dbm_record_decrement_refcount(&(ancestor->rb_data));
             }
         }
-        if (((++idx) % 100000) == 0)
+#endif
+        var_AUTO(ancestor, (struct rb_node *)item);
+        AVL_SET_NEXT(ancestor, *tree_recycle_bin);
+        *tree_recycle_bin = ancestor;
+        if (((++idx) % 10000000) == 0)
         {
 #ifdef WIN32
             fprintf(out_fh,
@@ -335,9 +243,8 @@ static inline void mark_and_sweep_old_states(
     TRACE("Finish mark-and-sweep cleanup for curr_depth=%lu\n",
         (unsigned long)curr_depth);
 #endif
-}
-
 #endif
+}
 
 #ifdef FCS_DBM_SINGLE_THREAD
 #define NUM_THREADS() 1
@@ -345,23 +252,10 @@ static inline void mark_and_sweep_old_states(
 #define NUM_THREADS() num_threads
 #endif
 
-#ifndef FCS_DBM_CACHE_ONLY
 #define DESTROY_STORE(instance)                                                \
     fc_solve_dbm_store_destroy((instance)->cache_store.store)
-#else
-#define DESTROY_STORE(instance)
-#endif
 
-#ifndef FCS_DBM_WITHOUT_CACHES
-#define DESTROY_CACHE(instance)                                                \
-    {                                                                          \
-        PRE_CACHE_OFFLOAD(instance);                                           \
-        cache_destroy(&(instance->cache_store.cache));                         \
-        DESTROY_STORE(instance);                                               \
-    }
-#else
 #define DESTROY_CACHE(instance) DESTROY_STORE(instance)
-#endif
 
 static inline void instance_increment(dbm_solver_instance *const instance)
 {
@@ -388,17 +282,15 @@ static inline void fcs_dbm__cache_store__init(
     const unsigned long pre_cache_max_count GCC_UNUSED,
     const unsigned long caches_delta GCC_UNUSED)
 {
-#ifndef FCS_DBM_WITHOUT_CACHES
-#ifndef FCS_DBM_CACHE_ONLY
-    pre_cache_init(&(cache_store->pre_cache), meta_alloc);
-#endif
-    cache_init(
-        &(cache_store->cache), pre_cache_max_count + caches_delta, meta_alloc);
-#endif
-#ifndef FCS_DBM_CACHE_ONLY
     fc_solve_dbm_store_init(
-        &(cache_store->store), dbm_store_path, &(common->tree_recycle_bin));
+        &(cache_store->store), dbm_store_path, &(common->tree_recycle_bin),
+#ifdef FCS_DBM__STORE_KEYS_ONLY
+        (common->do_not_yield_solution ? NULL : &(common->parent_lookup))
+#else
+        NULL
 #endif
+
+    );
 }
 
 typedef struct
@@ -489,46 +381,11 @@ static inline bool fcs_dbm__extract_common_from_argv(const int argc,
 
 static inline fcs_dbm_record *cache_store__has_key(
     fcs_dbm__cache_store__common *const cache_store,
-    fcs_encoded_state_buffer *const key, fcs_dbm_store_val parent)
+    fcs_encoded_state_buffer *const key, const fcs_encoded_state_buffer parent)
 {
-#ifndef FCS_DBM_WITHOUT_CACHES
-    if (cache_does_key_exist(&(cache_store->cache), key))
-    {
-        return NULL;
-    }
-#ifndef FCS_DBM_CACHE_ONLY
-    else if (pre_cache_does_key_exist(&(cache_store->pre_cache), key))
-    {
-        return NULL;
-    }
-    else if (fc_solve_dbm_store_does_key_exist(cache_store->store, key->s))
-    {
-        cache_insert(&(cache_store->cache), key, NULL, '\0');
-        return NULL;
-    }
-#endif
-    return ((fcs_dbm_record *)key);
-#else
-    return fc_solve_dbm_store_insert_key_value(
-        cache_store->store, key, parent, true);
-#endif
+    return fc_solve_dbm_store_insert_key_value(cache_store->store, key, parent);
 }
 
-#ifndef FCS_DBM_WITHOUT_CACHES
-static inline fcs_cache_key_info *cache_store__insert_key(
-    fcs_dbm__cache_store__common *const cache_store,
-    fcs_encoded_state_buffer *const key, fcs_dbm_record *const parent,
-    const fcs_fcc_move *const moves_to_parent, const uint8_t move GCC_UNUSED)
-{
-#ifndef FCS_DBM_CACHE_ONLY
-    pre_cache_insert(&(cache_store->pre_cache), key, &(parent->key));
-    return NULL;
-#else
-    return cache_insert(&(cache_store->cache), key, moves_to_parent, move);
-#endif
-}
-#endif
-
 #ifdef __cplusplus
 }
 #endif
