@@ -21,6 +21,7 @@ typedef struct
     const char *offload_dir_path;
 #endif
     dbm_instance_common_elems common;
+    fcs_dbm_record raw_found_sol;
 } dbm_solver_instance;
 
 #define CHECK_KEY_CALC_DEPTH() 0
@@ -29,7 +30,8 @@ typedef struct
 static inline void instance_init(dbm_solver_instance *const instance,
     const fcs_dbm_common_input *const inp,
     const unsigned long iters_delta_limit,
-    const unsigned long max_num_states_in_collection, FILE *const out_fh)
+    const unsigned long max_num_states_in_collection, FILE *const out_fh,
+    const bool do_not_yield_solution)
 {
     fc_solve_meta_compact_allocator_init(&(instance->meta_alloc));
 #ifdef FCS_DBM_USE_OFFLOADING_QUEUE
@@ -40,6 +42,23 @@ static inline void instance_init(dbm_solver_instance *const instance,
 #endif
     fcs_dbm__common_init(&(instance->common), iters_delta_limit,
         max_num_states_in_collection, inp->local_variant, out_fh);
+#ifdef FCS_DBM__STORE_KEYS_ONLY
+    instance->common.do_not_yield_solution = do_not_yield_solution;
+    if (!do_not_yield_solution)
+    {
+        /* initialize parent_lookup . */
+        char mypath[2000];
+        snprintf(mypath, COUNT(mypath), "%s/fcsdbm_rawdump.bin",
+            instance->offload_dir_path);
+        LAST(mypath) = '\0';
+
+        parent_lookup__create(&(instance->common.parent_lookup), mypath);
+    }
+    else
+    {
+        parent_lookup__create(&(instance->common.parent_lookup), NULL);
+    }
+#endif
     fcs_dbm__cache_store__init(&(instance->cache_store), &(instance->common),
         &(instance->meta_alloc), inp->dbm_store_path, inp->pre_cache_max_count,
         inp->caches_delta);
@@ -73,30 +92,20 @@ static inline void instance_destroy(dbm_solver_instance *const instance)
 static inline void instance_check_key(
     dbm_solver_thread *const thread GCC_UNUSED,
     dbm_solver_instance *const instance, const size_t key_depth GCC_UNUSED,
-    fcs_encoded_state_buffer *const key, fcs_dbm_record *const parent,
-    const uint8_t move GCC_UNUSED,
+    fcs_encoded_state_buffer *const key,
+    const fcs_encoded_state_buffer raw_parent, const uint8_t move GCC_UNUSED,
     const fcs_which_moves_bitmask *const which_irreversible_moves_bitmask
-        GCC_UNUSED
-#ifdef FCS_DBM_CACHE_ONLY
-    ,
-    const fcs_fcc_move *moves_to_parent
-#endif
-)
+        GCC_UNUSED)
 {
     fcs_dbm_record *token;
-    if ((token = cache_store__has_key(&instance->cache_store, key, parent)))
+    if ((token = cache_store__has_key(&instance->cache_store, key, raw_parent)))
     {
-#ifndef FCS_DBM_WITHOUT_CACHES
-        fcs_cache_key_info *cache_key = cache_store__insert_key(
-            &(instance->cache_store), key, parent, moves_to_parent, move);
-#endif
         ++instance->common.count_of_items_in_queue;
         ++instance->common.num_states_in_collection;
 
         instance_debug_out_state(instance, &(token->key));
 
-        fcs_offloading_queue__insert(
-            &(instance->queue), ((offloading_queue_item *)(&token)));
+        fcs_offloading_queue__insert(&(instance->queue), (token->key));
     }
 }
 
@@ -110,7 +119,9 @@ static void *instance_run_solver_thread(void *const void_arg)
 {
     fcs_dbm_queue_item physical_item;
     fcs_state_keyval_pair state;
-    fcs_dbm_record *token = NULL;
+    fcs_encoded_state_buffer null_parent;
+    fcs_init_encoded_state(&null_parent);
+    fcs_encoded_state_buffer token = null_parent;
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
 
     const_AUTO(thread, ((thread_arg *)void_arg)->thread);
@@ -145,10 +156,9 @@ static void *instance_run_solver_thread(void *const void_arg)
 
         if (instance->common.should_terminate == DONT_TERMINATE)
         {
-            if (fcs_offloading_queue__extract(
-                    &(instance->queue), (offloading_queue_item *)(&token)))
+            if (fcs_offloading_queue__extract(&(instance->queue), (&token)))
             {
-                physical_item.key = token->key;
+                physical_item.key = token;
                 item = &physical_item;
                 instance_increment(instance);
             }
@@ -187,7 +197,12 @@ static void *instance_run_solver_thread(void *const void_arg)
                     &derived_list_allocator, true))
             {
                 fcs_lock_lock(&instance->common.storage_lock);
-                fcs_dbm__found_solution(&(instance->common), token, item);
+                instance->raw_found_sol.key = token;
+                assert(dbm_lookup_parent(1, &instance->cache_store.store,
+                    instance->raw_found_sol.key,
+                    &instance->raw_found_sol.parent));
+                fcs_dbm__found_solution(
+                    &(instance->common), &instance->raw_found_sol);
                 fcs_lock_unlock(&instance->common.storage_lock);
                 break;
             }
@@ -197,17 +212,13 @@ static void *instance_run_solver_thread(void *const void_arg)
                 derived_iter = derived_iter->next)
             {
                 fcs_init_and_encode_state(delta_stater, local_variant,
-                    &(derived_iter->state), &(derived_iter->key));
+                    &(derived_iter->state),
+                    &(derived_iter->key_and_parent.key));
             }
 
-            instance_check_multiple_keys(
-                thread, instance, &(instance->cache_store),
-                &(instance->meta_alloc), &derived_list, 1
-#ifdef FCS_DBM_CACHE_ONLY
-                ,
-                item->moves_to_key
-#endif
-            );
+            instance_check_multiple_keys(thread, instance,
+                &(instance->cache_store), &(instance->meta_alloc),
+                &derived_list, 1);
 
             fcs_derived_state_list__recycle(
                 &derived_list_recycle_bin, &derived_list);
@@ -273,14 +284,10 @@ static bool populate_instance_with_intermediate_input_line(
         delta, local_variant, &(running_state), &running_key);
     fcs_dbm_record *running_parent = NULL;
 
-#ifdef FCS_DBM_CACHE_ONLY
-    fcs_fcc_move *running_moves = NULL;
-    cache_store__insert_key(&(instance->cache_store), &(running_key),
-        &running_parent, running_moves, '\0');
-#else
+    fcs_encoded_state_buffer null_parent;
+    fcs_init_encoded_state(&null_parent);
     running_parent = fc_solve_dbm_store_insert_key_value(
-        instance->cache_store.store, &(running_key), running_parent, true);
-#endif
+        instance->cache_store.store, &(running_key), null_parent);
     ++instance->common.num_states_in_collection;
 
     unsigned int hex_digits;
@@ -347,22 +354,12 @@ static bool populate_instance_with_intermediate_input_line(
         fcs_init_and_encode_state(
             delta, local_variant, &(running_state), &(running_key));
 
-#ifndef FCS_DBM_WITHOUT_CACHES
-        var_AUTO(cache_ret,
-            cache_store__insert_key(&(instance->cache_store), &(running_key),
-                &running_parent, running_moves, move));
-        if (cache_ret)
-        {
-            running_moves = cache_ret->moves_to_key;
-        }
-#else
-        token = fc_solve_dbm_store_insert_key_value(
-            instance->cache_store.store, &(running_key), running_parent, true);
+        token = fc_solve_dbm_store_insert_key_value(instance->cache_store.store,
+            &(running_key), running_parent->parent);
         if (!token)
         {
             return false;
         }
-#endif
         ++instance->common.num_states_in_collection;
 
         // We need to do the round-trip from encoding back to decoding, because
@@ -381,8 +378,7 @@ static bool populate_instance_with_intermediate_input_line(
                    "that with all states applied.\n",
             line_num);
     }
-    fcs_offloading_queue__insert(
-        &(instance->queue), (const offloading_queue_item *)(&token));
+    fcs_offloading_queue__insert(&(instance->queue), (token->key));
     ++instance->common.count_of_items_in_queue;
 
     return true;
@@ -422,6 +418,12 @@ static void instance_run_all_threads(dbm_solver_instance *const instance,
     }
 #endif
 
+#ifdef FCS_DBM__STORE_KEYS_ONLY
+    if (!instance->common.do_not_yield_solution)
+    {
+        parent_lookup__finish_writing(&(instance->common.parent_lookup));
+    }
+#endif
     dbm__free_threads(instance, num_threads, threads, free_thread);
 }
 
@@ -431,7 +433,7 @@ static bool handle_and_destroy_instance_solution(
 {
     FILE *const out_fh = instance->common.out_fh;
     bool ret = false;
-    fcs_dbm_record *token;
+    fcs_encoded_state_buffer token;
 #ifdef DEBUG_OUT
     fcs_dbm_variant_type local_variant = instance->common.variant;
 #endif
@@ -441,7 +443,8 @@ static bool handle_and_destroy_instance_solution(
 
     if (instance->common.queue_solution_was_found)
     {
-        trace_solution(instance, out_fh, delta);
+        trace_solution(
+            1, &instance->cache_store.store, instance, out_fh, delta);
         ret = true;
     }
     else if (instance->common.should_terminate != DONT_TERMINATE)
@@ -453,10 +456,9 @@ static bool handle_and_destroy_instance_solution(
             fcs_dbm_queue_item physical_item;
             fcs_dbm_queue_item *const item = &physical_item;
 
-            while (fcs_offloading_queue__extract(
-                &(instance->queue), (offloading_queue_item *)(&token)))
+            while (fcs_offloading_queue__extract(&(instance->queue), (&token)))
             {
-                physical_item.key = token->key;
+                physical_item.key = token;
 
 #ifdef FCS_DEBONDT_DELTA_STATES
                 for (size_t i = 0; i < COUNT(item->key.s); i++)
@@ -473,20 +475,14 @@ static bool handle_and_destroy_instance_solution(
                 fprintf(out_fh, "%s", "|");
                 fflush(out_fh);
 
-#ifdef FCS_DBM_CACHE_ONLY
-                fcs_fcc_move *move_ptr = item->moves_to_key;
-                if (move_ptr)
-                {
-                    while (*(move_ptr))
-                    {
-                        fprintf(out_fh, "%.2X,", *(move_ptr));
-                        move_ptr++;
-                    }
-                }
-#else
                 size_t trace_num;
                 fcs_encoded_state_buffer *trace;
-                calc_trace(token, &trace, &trace_num);
+                fcs_dbm_record raw_found;
+                raw_found.key = token;
+                assert(dbm_lookup_parent(1, &instance->cache_store.store,
+                    raw_found.key, &raw_found.parent));
+                calc_trace(1, &instance->cache_store.store, &raw_found, &trace,
+                    &trace_num);
 
 // We stop at 1 because the deepest state does not contain a move (as it is the
 // ultimate state).
@@ -499,7 +495,6 @@ static bool handle_and_destroy_instance_solution(
                 }
 #undef PENULTIMATE_DEPTH
                 free(trace);
-#endif
                 fprintf(out_fh, "\n");
                 fflush(out_fh);
 #ifdef DEBUG_OUT
@@ -540,8 +535,9 @@ int main(int argc, char *argv[])
     FILE *intermediate_in_fh = NULL;
     bool skip_queue_output = false;
     DECLARE_IND_BUF_T(init_indirect_stacks_buffer)
-    const char *param;
     fcs_dbm_common_input inp = fcs_dbm_common_input_init;
+    const char *param;
+    bool do_not_yield_solution = false;
 
     int arg;
     for (arg = 1; arg < argc; arg++)
@@ -561,6 +557,10 @@ int main(int argc, char *argv[])
         {
             intermediate_input_filename = param;
         }
+        else if ((param = TRY_P("--do-not-yield-solution")))
+        {
+            do_not_yield_solution = ((!strcmp(param, "0")) ? false : true);
+        }
         else
         {
             break;
@@ -614,9 +614,10 @@ int main(int argc, char *argv[])
         dbm_solver_instance queue_instance;
         dbm_solver_instance limit_instance;
 
-        instance_init(&queue_instance, &inp, ULONG_MAX, ULONG_MAX, out_fh);
-        instance_init(
-            &limit_instance, &inp, inp.iters_delta_limit, ULONG_MAX, out_fh);
+        instance_init(&queue_instance, &inp, ULONG_MAX, ULONG_MAX, out_fh,
+            do_not_yield_solution);
+        instance_init(&limit_instance, &inp, inp.iters_delta_limit, ULONG_MAX,
+            out_fh, do_not_yield_solution);
 
         bool found_line;
         do
@@ -658,7 +659,8 @@ int main(int argc, char *argv[])
 
                     if (limit_instance.common.queue_solution_was_found)
                     {
-                        trace_solution(&limit_instance, out_fh, &delta);
+                        trace_solution(1, &limit_instance.cache_store.store,
+                            &limit_instance, out_fh, &delta);
                         skip_queue_output = true;
                         queue_solution_was_found = true;
                     }
@@ -725,30 +727,25 @@ int main(int argc, char *argv[])
     else
     {
         dbm_solver_instance instance;
-        instance_init(
-            &instance, &inp, inp.iters_delta_limit, ULONG_MAX, out_fh);
+        instance_init(&instance, &inp, inp.iters_delta_limit, ULONG_MAX, out_fh,
+            do_not_yield_solution);
         fcs_encoded_state_buffer *key_ptr;
-#define KEY_PTR() (key_ptr)
         fcs_encoded_state_buffer parent_state_enc;
 
         key_ptr = &(instance.common.first_key);
         fcs_init_and_encode_state(
-            &delta, local_variant, &(init_state), KEY_PTR());
+            &delta, local_variant, &(init_state), key_ptr);
         // The NULL parent_state_enc and move for indicating this is the
         // initial state.
         fcs_init_encoded_state(&(parent_state_enc));
 
         fcs_dbm_record *token;
-#ifndef FCS_DBM_WITHOUT_CACHES
-        cache_store__insert_key(
-            &(instance.cache_store), KEY_PTR(), &parent_state_enc, NULL, '\0');
-#else
+        fcs_encoded_state_buffer null_parent;
+        fcs_init_encoded_state(&null_parent);
         token = fc_solve_dbm_store_insert_key_value(
-            instance.cache_store.store, KEY_PTR(), NULL, true);
-#endif
+            instance.cache_store.store, key_ptr, null_parent);
 
-        fcs_offloading_queue__insert(
-            &(instance.queue), (const offloading_queue_item *)&token);
+        fcs_offloading_queue__insert(&(instance.queue), token->key);
         ++instance.common.num_states_in_collection;
         ++instance.common.count_of_items_in_queue;
 
