@@ -14,6 +14,7 @@
 // In addition, this solver implements the scheme in:
 // https://groups.yahoo.com/neo/groups/fc-solve-discuss/conversations/topics/1135
 
+#define IN_DEPTH_DBM_SOLVER_dot_c 1
 #include "dbm_solver_head.h"
 
 typedef struct
@@ -23,7 +24,6 @@ typedef struct
     fcs_offloading_queue queue;
 } fcs_dbm_collection_by_depth;
 
-#define MAX_FCC_DEPTH (RANK_KING * 4 * DECKS_NUM * 2)
 typedef size_t fcs_batch_size;
 typedef struct
 {
@@ -33,6 +33,7 @@ typedef struct
     size_t curr_depth;
     dbm_instance_common_elems common;
     fcs_batch_size max_batch_size;
+    fcs_dbm_record raw_found_sol;
 } dbm_solver_instance;
 
 #define CHECK_KEY_CALC_DEPTH()                                                 \
@@ -41,13 +42,30 @@ typedef struct
 #include "dbm_procs.h"
 static inline void instance_init(dbm_solver_instance *const instance,
     const fcs_dbm_common_input *const inp, const fcs_batch_size max_batch_size,
-    FILE *const out_fh)
+    FILE *const out_fh, const bool do_not_yield_solution)
 {
     instance->max_batch_size = max_batch_size;
     instance->curr_depth = 0;
     instance->offload_dir_path = inp->offload_dir_path;
     fcs_dbm__common_init(&(instance->common), inp->iters_delta_limit,
         inp->max_num_states_in_collection, inp->local_variant, out_fh);
+#ifdef FCS_DBM__STORE_KEYS_ONLY
+    instance->common.do_not_yield_solution = do_not_yield_solution;
+    if (!do_not_yield_solution)
+    {
+        /* initialize parent_lookup . */
+        char mypath[2000];
+        snprintf(mypath, COUNT(mypath), "%s/fcsdbm_rawdump.bin",
+            instance->offload_dir_path);
+        LAST(mypath) = '\0';
+
+        parent_lookup__create(&(instance->common.parent_lookup), mypath);
+    }
+    else
+    {
+        parent_lookup__create(&(instance->common.parent_lookup), NULL);
+    }
+#endif
 
     for (int depth = 0; depth < MAX_FCC_DEPTH; depth++)
     {
@@ -114,7 +132,7 @@ static void *instance_run_solver_thread(void *const void_arg)
     TRACE("%s\n", "instance_run_solver_thread start");
 
     const_AUTO(coll, &(instance->colls_by_depth[instance->curr_depth]));
-    fcs_dbm_record *tokens[max_batch_size];
+    fcs_encoded_state_buffer tokens[max_batch_size];
     fcs_derived_state *derived_lists[max_batch_size];
     while (true)
     {
@@ -139,8 +157,8 @@ static void *instance_run_solver_thread(void *const void_arg)
             {
                 for (; batch_size < max_batch_size; ++batch_size)
                 {
-                    if (fcs_offloading_queue__extract(&(coll->queue),
-                            (offloading_queue_item *)(&tokens[batch_size])))
+                    if (fcs_offloading_queue__extract(
+                            &(coll->queue), (&tokens[batch_size])))
                     {
                         derived_lists[batch_size] = NULL;
                         instance_increment(instance);
@@ -179,18 +197,26 @@ static void *instance_run_solver_thread(void *const void_arg)
             const_AUTO(token, tokens[batch_i]);
             // Handle the item in "token".
             fc_solve_delta_stater_decode_into_state(
-                delta_stater, token->key.s, &state, indirect_stacks_buffer);
+                delta_stater, token.s, &state, indirect_stacks_buffer);
             FCS__OUTPUT_STATE(out_fh, "", &(state.s), &locs);
 
             if (instance_solver_thread_calc_derived_states(local_variant,
                     &state, token, &derived_lists[batch_i],
                     &derived_list_recycle_bin, &derived_list_allocator, true))
             {
-                fcs_dbm_queue_item physical_item;
-                physical_item.key = token->key;
                 fcs_lock_lock(&instance->common.storage_lock);
+                instance->raw_found_sol.key = token;
+                fcs_dbm_store stores_by_depth[MAX_FCC_DEPTH];
+                for (size_t i = 0; i < MAX_FCC_DEPTH; ++i)
+                {
+                    stores_by_depth[i] =
+                        instance->colls_by_depth[i].cache_store.store;
+                }
+                assert(dbm_lookup_parent(MAX_FCC_DEPTH, stores_by_depth,
+                    instance->raw_found_sol.key,
+                    &instance->raw_found_sol.parent));
                 fcs_dbm__found_solution(
-                    &(instance->common), token, &physical_item);
+                    &(instance->common), &instance->raw_found_sol);
                 fcs_condvar_broadcast(&(instance->monitor));
                 fcs_lock_unlock(&instance->common.storage_lock);
                 goto thread_end;
@@ -204,17 +230,13 @@ static void *instance_run_solver_thread(void *const void_arg)
                 derived_iter = derived_iter->next)
             {
                 fcs_init_and_encode_state(delta_stater, local_variant,
-                    &(derived_iter->state), &(derived_iter->key));
+                    &(derived_iter->state),
+                    &(derived_iter->key_and_parent.key));
             }
         }
 
         instance_check_multiple_keys(thread, instance, &(coll->cache_store),
-            &(thread->thread_meta_alloc), derived_lists, batch_size
-#ifdef FCS_DBM_CACHE_ONLY
-            ,
-            item->moves_to_key
-#endif
-        );
+            &(thread->thread_meta_alloc), derived_lists, batch_size);
 
         for (fcs_batch_size batch_i = 0; batch_i < batch_size; ++batch_i)
         {
@@ -235,27 +257,18 @@ thread_end:
 static inline void instance_check_key(
     dbm_solver_thread *const thread GCC_UNUSED,
     dbm_solver_instance *const instance, const size_t key_depth,
-    fcs_encoded_state_buffer *const key, fcs_dbm_record *const parent,
+    fcs_encoded_state_buffer *const key,
+    const fcs_encoded_state_buffer raw_parent,
     const unsigned char move GCC_UNUSED,
     const fcs_which_moves_bitmask *const which_irreversible_moves_bitmask
-        GCC_UNUSED
-#ifdef FCS_DBM_CACHE_ONLY
-    ,
-    const fcs_fcc_move *moves_to_parent
-#endif
-)
+        GCC_UNUSED)
 {
     const_AUTO(coll, &(instance->colls_by_depth[key_depth]));
     fcs_dbm_record *token;
 
-    if ((token = cache_store__has_key(&coll->cache_store, key, parent)))
+    if ((token = cache_store__has_key(&coll->cache_store, key, raw_parent)))
     {
-#ifndef FCS_DBM_WITHOUT_CACHES
-        fcs_cache_key_info *cache_key = cache_store__insert_key(
-            &(instance->cache_store), key, parent, moves_to_parent, move);
-#endif
-        fcs_offloading_queue__insert(
-            &(coll->queue), (const offloading_queue_item *)(&token));
+        fcs_offloading_queue__insert(&(coll->queue), (token->key));
 
         ++instance->common.count_of_items_in_queue;
         ++instance->common.num_states_in_collection;
@@ -283,6 +296,12 @@ static void instance_run_all_threads(dbm_solver_instance *const instance,
         ++instance->curr_depth;
     }
 
+#ifdef FCS_DBM__STORE_KEYS_ONLY
+    if (!instance->common.do_not_yield_solution)
+    {
+        parent_lookup__finish_writing(&(instance->common.parent_lookup));
+    }
+#endif
     dbm__free_threads(instance, num_threads, threads, free_thread);
 }
 
@@ -294,6 +313,7 @@ int main(int argc, char *argv[])
     fcs_dbm_common_input inp = fcs_dbm_common_input_init;
     fcs_batch_size max_batch_size = 1;
     const char *param;
+    bool do_not_yield_solution = false;
 
     int real_arg;
     int *arg = &real_arg;
@@ -314,6 +334,10 @@ int main(int argc, char *argv[])
         {
             out_filename = param;
         }
+        else if ((param = TRY_PARAM("--do-not-yield-solution")))
+        {
+            do_not_yield_solution = ((!strcmp(param, "0")) ? false : true);
+        }
         else
         {
             break;
@@ -344,28 +368,25 @@ int main(int argc, char *argv[])
     fc_solve_delta_stater_init(&delta, local_variant, &init_state.s, STACKS_NUM,
         FREECELLS_NUM PASS_ON_NOT_FC_ONLY(CALC_SEQUENCES_ARE_BUILT_BY()));
 
-#define KEY_PTR() (key_ptr)
     dbm_solver_instance instance;
-    instance_init(&instance, &inp, max_batch_size, out_fh);
+    instance_init(
+        &instance, &inp, max_batch_size, out_fh, do_not_yield_solution);
 
     fcs_encoded_state_buffer *const key_ptr = &(instance.common.first_key);
-    fcs_init_and_encode_state(&delta, local_variant, &init_state, KEY_PTR());
+    fcs_init_and_encode_state(&delta, local_variant, &init_state, key_ptr);
 
     // The NULL parent for indicating this is the initial state.
     fcs_encoded_state_buffer parent_state_enc;
     fcs_init_encoded_state(&(parent_state_enc));
 
     fcs_dbm_record *token;
-#ifndef FCS_DBM_WITHOUT_CACHES
-    cache_store__insert_key(
-        &(instance.cache_store), KEY_PTR(), &parent_state_enc, NULL, '\0');
-#else
+    fcs_encoded_state_buffer null_parent;
+    fcs_init_encoded_state(&null_parent);
     token = fc_solve_dbm_store_insert_key_value(
-        instance.colls_by_depth[0].cache_store.store, KEY_PTR(), NULL, true);
-#endif
+        instance.colls_by_depth[0].cache_store.store, key_ptr, null_parent);
 
-    fcs_offloading_queue__insert(&(instance.colls_by_depth[0].queue),
-        (const offloading_queue_item *)(&token));
+    fcs_offloading_queue__insert(
+        &(instance.colls_by_depth[0].queue), (token->key));
     ++instance.common.num_states_in_collection;
     ++instance.common.count_of_items_in_queue;
 
