@@ -78,6 +78,39 @@ static inline enum rb_color rb_set_color(
 }
 #endif
 
+// Should make the runtime faster. Enable at your own risk.
+//
+// Trust me: every problem in computer science may be solved by an indirection,
+// but those indirections are expensive. Pointer chasing is just about the most
+// expensive thing you can do on modern CPUs.
+// --- Linus Torvalds ,
+// https://www.shlomifish.org/humour/fortunes/show.cgi?id=linus-torvalds-about-indirections
+//
+#define AVL__INLINE_COMPARISON_FUNC 1
+
+#ifdef AVL__INLINE_COMPARISON_FUNC
+
+#include "dbm_kaztree_compare.h"
+
+#ifdef AVL_with_rb_param
+#define AVL__COMPARE(tree, rb_a, rb_b, rb_context)                             \
+    fcs_dbm__compare_records(rb_a, rb_b, rb_context)
+#else
+#define AVL__COMPARE(tree, rb_a, rb_b, rb_context)                             \
+    fcs_dbm__compare_records__noctx(rb_a, rb_b)
+#endif
+
+#else
+
+#ifdef AVL_with_rb_param
+#define AVL__COMPARE(tree, rb_a, rb_b, rb_context)                             \
+    tree->rb_compare(rb_a, rb_b, rb_context)
+#else
+#define AVL__COMPARE(tree, rb_a, rb_b, rb_context) tree->rb_compare(rb_a, rb_b)
+#endif
+
+#endif
+
 /* Creates and returns a new table
    with comparison function |compare| using parameter |param|
    and memory allocator |allocator|.
@@ -119,12 +152,7 @@ void *rb_find(const struct rb_table *tree, const void *item)
     assert(tree != NULL && item != NULL);
     for (p = TREE_AVL_ROOT(tree); p != NULL;)
     {
-        int cmp = tree->rb_compare(item, NODE_DATA_PTR(p)
-#ifdef AVL_with_rb_param
-                                             ,
-            tree->rb_param
-#endif
-        );
+        int cmp = AVL__COMPARE(tree, item, NODE_DATA_PTR(p), tree->rb_param);
 
         if (cmp < 0)
             p = rb_process_link(p->rb_mylink[0]);
@@ -141,7 +169,8 @@ void *rb_find(const struct rb_table *tree, const void *item)
    If a duplicate item is found in the tree,
    returns a pointer to the duplicate without inserting |item|.
    Returns |NULL| in case of memory allocation failure. */
-avl_key_type *rb_probe(struct rb_table *tree, void *item)
+avl_key_type *rb_probe(
+    struct rb_table *tree, void *item, bool *was_item_inserted_now)
 {
     struct rb_node *pa[RB_MAX_HEIGHT]; /* Nodes on stack. */
     unsigned char da[RB_MAX_HEIGHT];   /* Directions moved from stack nodes. */
@@ -158,19 +187,18 @@ avl_key_type *rb_probe(struct rb_table *tree, void *item)
     for (p = TREE_AVL_ROOT(tree); p != NULL;
         p = rb_process_link(p->rb_mylink[da[k - 1]]))
     {
-        int cmp = tree->rb_compare(item, NODE_DATA_PTR(p)
-#ifdef AVL_with_rb_param
-                                             ,
-            tree->rb_param
-#endif
-        );
+        int cmp = AVL__COMPARE(tree, item, NODE_DATA_PTR(p), tree->rb_param);
         if (cmp == 0)
+        {
+            *was_item_inserted_now = false;
             return NODE_DATA_PTR(p);
+        }
 
         pa[k] = p;
         da[k++] = cmp > 0;
     }
 
+    *was_item_inserted_now = true;
     if ((n = *(tree->rb_recycle_bin)) != NULL)
     {
         *(tree->rb_recycle_bin) = AVL_NEXT(n);
@@ -264,32 +292,33 @@ avl_key_type *rb_probe(struct rb_table *tree, void *item)
     }
     rb_set_color(TREE_AVL_ROOT(tree), RB_BLACK);
 
-#ifndef FCS_DBM__VAL_IS_ANCESTOR
     return NODE_DATA_PTR(n);
-#else
-    return NULL;
-#endif
 }
 
-#ifdef FCS_DBM__VAL_IS_ANCESTOR
-#else
+#if 0
+
+// an updated "magic" flag.
+#if defined(FCS_DBM__STORE_KEYS_ONLY)
+#define FCS_DBM__GET_A_avl_key_type_COMPARISON_WITH_key_ONLY_TO_WORK
+#endif
+
 /* Inserts |item| into |table|.
    Returns |NULL| if |item| was successfully inserted
    or if a memory allocation error occurred.
    Otherwise, returns the duplicate item. */
 void *rb_insert(struct rb_table *table, void *item)
 {
-    avl_key_type *p = rb_probe(table, item);
-#ifndef FCS_DBM__VAL_IS_ANCESTOR
+    bool was_item_inserted_now;
+    avl_key_type *p = rb_probe(table, item, &was_item_inserted_now);
+#ifdef FCS_DBM__GET_A_avl_key_type_COMPARISON_WITH_key_ONLY_TO_WORK
+    void *ret = p == NULL ? NULL : AVL_KEY_PTR_PTR(p);
+#else
     void *ret = p == NULL || (AVL_KEY_EQUAL_TO_PTR(*p, item))
                     ? NULL
                     : AVL_KEY_PTR_PTR(p);
-#else
-    void *ret = p == NULL ? NULL : AVL_KEY_PTR_PTR(p);
 #endif
     return ret;
 }
-#endif
 
 /* Inserts |item| into |table|, replacing any duplicate item.
    Returns |NULL| if |item| was inserted without replacing a duplicate,
@@ -297,7 +326,8 @@ void *rb_insert(struct rb_table *table, void *item)
    Otherwise, returns the item that was replaced. */
 void *rb_replace(struct rb_table *table, void *item)
 {
-    avl_key_type *p = rb_probe(table, item);
+    bool was_item_inserted_now;
+    avl_key_type *p = rb_probe(table, item, &was_item_inserted_now);
     if (p == NULL || AVL_KEY_EQUAL_TO_PTR(*p, item))
         return NULL;
     else
@@ -525,6 +555,7 @@ void *rb_delete(struct rb_table *tree, const void *item)
     tree->rb_generation++;
     return new_item;
 }
+#endif
 
 /* Refreshes the stack of parent pointers in |trav|
    and updates its generation number. */
@@ -536,7 +567,7 @@ static void trav_refresh(struct rb_traverser *trav)
 
     if (trav->rb_node != NULL)
     {
-        rb_comparison_func *cmp = trav->rb_table->rb_compare;
+        var_AUTO(tree, trav->rb_table);
 #ifdef AVL_with_rb_param
         void *param = trav->rb_table->rb_param;
 #endif
@@ -551,12 +582,8 @@ static void trav_refresh(struct rb_traverser *trav)
 
             trav->rb_stack[trav->rb_height++] = i;
             i = rb_process_link(
-                i->rb_mylink[cmp(NODE_DATA_PTR(node), NODE_DATA_PTR(i)
-#ifdef AVL_with_rb_param
-                                                          ,
-                                 param
-#endif
-                                 ) > 0]);
+                i->rb_mylink[(AVL__COMPARE(tree, NODE_DATA_PTR(node),
+                                 NODE_DATA_PTR(i), param)) > 0]);
         }
     }
 }
@@ -638,12 +665,7 @@ void *rb_t_find(struct rb_traverser *trav, struct rb_table *tree, void *item)
     trav->rb_generation = tree->rb_generation;
     for (p = TREE_AVL_ROOT(tree); p != NULL; p = q)
     {
-        int cmp = tree->rb_compare(item, NODE_DATA_PTR(p)
-#ifdef AVL_with_rb_param
-                                             ,
-            tree->rb_param
-#endif
-        );
+        int cmp = AVL__COMPARE(tree, item, NODE_DATA_PTR(p), tree->rb_param);
 
         if (cmp < 0)
             q = rb_process_link(p->rb_mylink[0]);
@@ -677,7 +699,8 @@ void *rb_t_insert(struct rb_traverser *trav, struct rb_table *tree, void *item)
 
     assert(trav != NULL && tree != NULL && item != NULL);
 
-    p = rb_probe(tree, item);
+    bool was_item_inserted_now;
+    p = rb_probe(tree, item, &was_item_inserted_now);
     if (p != NULL)
     {
         trav->rb_table = tree;
