@@ -37,7 +37,7 @@ static void print_help(void)
 #endif
 
 static fcs_iters_int total_num_iters = 0;
-static long long total_num_finished_boards = 0;
+static fc_solve_ms_deal_idx_type total_num_finished_boards = 0;
 
 #define READ_FD 0
 #define WRITE_FD 1
@@ -48,37 +48,32 @@ typedef struct
 
 typedef struct
 {
-    long long board_num, quota_end;
+    fc_solve_ms_deal_idx_type board_num, quota_end;
 } request_type;
 
 typedef struct
 {
     fcs_iters_int num_iters;
-    long long num_finished_boards;
+    fc_solve_ms_deal_idx_type num_finished_boards;
 } response_type;
 
-static inline void write_request(const long long end_board,
-    const long long board_num_step, long long *const next_board_num_ptr,
+static inline void write_request(const fc_solve_ms_deal_idx_type end_board,
+    const fc_solve_ms_deal_idx_type board_num_step,
+    fc_solve_ms_deal_idx_type *const next_board_num_ptr,
     const fcs_worker *const worker)
 {
-    request_type req;
     if ((*next_board_num_ptr) > end_board)
     {
-        /* We only absolutely need to initialize .board_num here, but the
-         * Coverity Scan scanner complains about quota_end being uninitialized
-         * when passed to write() so we initialize it here as well.
-         * */
-        req = (request_type){.board_num = -1, .quota_end = -1};
+        close(worker->parent_to_child_pipe[WRITE_FD]);
+        return;
     }
-    else
+    request_type req;
+    req.board_num = *(next_board_num_ptr);
+    if (((*next_board_num_ptr) += board_num_step) > end_board)
     {
-        req.board_num = *(next_board_num_ptr);
-        if (((*next_board_num_ptr) += board_num_step) > end_board)
-        {
-            (*next_board_num_ptr) = end_board + 1;
-        }
-        req.quota_end = (*next_board_num_ptr) - 1;
+        (*next_board_num_ptr) = end_board + 1;
     }
+    req.quota_end = (*next_board_num_ptr) - 1;
     const_AUTO(
         ret, write(worker->parent_to_child_pipe[WRITE_FD], &req, sizeof(req)));
     if (ret != sizeof(req))
@@ -88,8 +83,9 @@ static inline void write_request(const long long end_board,
 }
 
 static inline void transaction(const fcs_worker *const worker,
-    const int read_fd, const long long end_board,
-    const long long board_num_step, long long *const next_board_num_ptr)
+    const int read_fd, const fc_solve_ms_deal_idx_type end_board,
+    const fc_solve_ms_deal_idx_type board_num_step,
+    fc_solve_ms_deal_idx_type *const next_board_num_ptr)
 {
     response_type response;
     if (read(read_fd, &response, sizeof(response)) <
@@ -109,11 +105,12 @@ static inline int read_fd(const fcs_worker *const worker)
 }
 
 static inline int range_solvers_main(int argc, char *argv[], int arg,
-    long long next_board_num, const long long end_board,
-    const long long stop_at)
+    fc_solve_ms_deal_idx_type next_board_num,
+    const fc_solve_ms_deal_idx_type end_board,
+    const fc_solve_ms_deal_idx_type stop_at)
 {
     size_t num_workers = 3;
-    long long board_num_step = 1;
+    fc_solve_ms_deal_idx_type board_num_step = 1;
     for (; arg < argc; ++arg)
     {
         const char *param;
@@ -123,7 +120,7 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
         }
         else if ((param = TRY_P("--worker-step")))
         {
-            board_num_step = atoll(param);
+            board_num_step = fcs_str2msdeal(param);
         }
         else
         {
@@ -154,17 +151,17 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
             exit_error("Fork for worker No. %lu failed! Exiting.\n",
                 (unsigned long)idx);
 
-        case 0:
-        {
-            /* I'm the child. */
+        case 0: {
+            // I'm the child.
             const_AUTO(w, workers[idx]);
             close(w.parent_to_child_pipe[WRITE_FD]);
             close(w.child_to_parent_pipe[READ_FD]);
-            /* I'm one of the slaves */
+            // I'm one of the workers
             request_type req;
+            fcs_state_string state_string;
+            get_board__setup_string(state_string);
             while (read(w.parent_to_child_pipe[READ_FD], &req, sizeof(req)) ==
-                       sizeof(req) &&
-                   req.board_num != -1)
+                   sizeof(req))
             {
                 response_type response = {
                     .num_iters = 0,
@@ -172,8 +169,8 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
                 };
                 for (; req.board_num <= req.quota_end; ++req.board_num)
                 {
-                    range_solvers__solve(
-                        instance, req.board_num, &response.num_iters);
+                    range_solvers__solve(state_string, instance, req.board_num,
+                        &response.num_iters);
                     freecell_solver_user_recycle(instance);
                 }
                 if (sizeof(response) != write(w.child_to_parent_pipe[WRITE_FD],
@@ -182,7 +179,6 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
                     abort();
                 }
             }
-            /* Cleanup */
             freecell_solver_user_free(instance);
             close(w.child_to_parent_pipe[WRITE_FD]);
             close(w.parent_to_child_pipe[READ_FD]);
@@ -190,7 +186,7 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
         }
 
         default:
-            /* I'm the parent. */
+            // I'm the parent.
             close(workers[idx].parent_to_child_pipe[READ_FD]);
             close(workers[idx].child_to_parent_pipe[WRITE_FD]);
             break;
@@ -199,7 +195,7 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
 
     freecell_solver_user_free(instance);
 
-/* I'm the master. */
+// I'm the master.
 #ifdef USE_EPOLL
     const int epollfd = epoll_create1(0);
     if (epollfd == -1)
@@ -237,9 +233,10 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
 #ifndef USE_EPOLL
     ++mymax;
 #endif
-    const long long total_num_boards_to_check = end_board - next_board_num + 1;
+    const fc_solve_ms_deal_idx_type total_num_boards_to_check =
+        end_board - next_board_num + 1;
 
-    long long next_milestone = stop_at;
+    fc_solve_ms_deal_idx_type next_milestone = stop_at;
     for (size_t idx = 0; idx < num_workers; ++idx)
     {
         write_request(
@@ -270,7 +267,7 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
         }
 #else
         fd_set readers = initial_readers;
-        /* I'm the master. */
+        // I'm the master.
         const int select_ret = select(mymax, &readers, NULL, NULL, NULL);
 
         if (select_ret == -1)
@@ -285,8 +282,7 @@ static inline int range_solvers_main(int argc, char *argv[], int arg,
 
                 if (FD_ISSET(fd, &readers))
                 {
-                    /* FD_ISSET can be set on EOF, so we check if
-                     * read failed. */
+                    // FD_ISSET can be set on EOF, so we check if read failed.
                     transaction(&(workers[idx]), fd, end_board, board_num_step,
                         &next_board_num);
                 }
