@@ -11,7 +11,7 @@
 #include "freecell-solver/fcs_conf.h"
 
 #ifdef FCS_DBM_USE_HASH
-#include "dbm_hashtable.h"
+#error FCS_DBM_USE_HASH
 #else
 
 #ifdef FCS_DBM_USE_LIBAVL
@@ -23,8 +23,6 @@ typedef void *dict_key_t;
 #define fc_solve_kaz_tree_create(comparator, context, meta, recycle_bin_ptr)   \
     rb_create(comparator, context, meta, recycle_bin_ptr)
 #define fc_solve_kaz_tree_lookup_value(tree, value) rb_find(tree, value)
-#define fc_solve_kaz_tree_delete_by_value(tree, value) rb_delete(tree, value)
-#define fc_solve_kaz_tree_alloc_insert(tree, value) rb_insert(tree, value)
 
 #else
 
