@@ -1,12 +1,10 @@
-/*
- * This file is part of patsolve. It is subject to the license terms in
- * the LICENSE file found in the top-level directory of this distribution
- * and at https://github.com/shlomif/patsolve/blob/master/LICENSE . No
- * part of patsolve, including this file, may be copied, modified, propagated,
- * or distributed except according to the terms contained in the COPYING file.
- *
- * Copyright (c) 2002 Tom Holroyd
- */
+// This file is part of patsolve. It is subject to the license terms in
+// the LICENSE file found in the top-level directory of this distribution
+// and at https://github.com/shlomif/patsolve/blob/master/LICENSE . No
+// part of patsolve, including this file, may be copied, modified, propagated,
+// or distributed except according to the terms contained in the COPYING file.
+//
+// Copyright (c) 2002 Tom Holroyd
 
 /* Solve patience games.  Prioritized breadth-first search.  Simple breadth-
 first uses exponential memory.  Here the work queue is kept sorted to give
@@ -23,7 +21,7 @@ many
 threads running through the game tree starting from the queued positions.
 The num_childs element keeps track of descendents, and when there are none left
 in the parent we can free it too after solve() returns and we get called
-recursively (rec == TRUE). */
+recursively (rec == true). */
 
 /* We don't really free anything here, we just push it onto a
    freelist (using the queue member), so we can use it again later.
@@ -54,12 +52,12 @@ static inline void free_position_recursive(
 }
 
 /* Like strcpy() but returns the length of the string. */
-static inline int strecpy(char *dest, const char *src)
+static inline int strecpy(fcs_card *dest, const fcs_card *src)
 {
     int i = 0;
     while ((*dest++ = *src++) != '\0')
     {
-        i++;
+        ++i;
     }
 
     return i;
@@ -101,7 +99,7 @@ static inline void unpack_position(
         unsigned char c = 0;
         unsigned char *p =
             (unsigned char *)(pos->node) + sizeof(fcs_pats__tree);
-        bool k = FALSE;
+        bool k = false;
         for (int w = 0; w < LOCAL_STACKS_NUM; w++)
         {
             int i;
@@ -120,8 +118,8 @@ static inline void unpack_position(
             soft_thread->current_pos.stack_ids[w] = i;
             var_AUTO(l, soft_thread->bucket_from_pile_lookup[i]);
             var_AUTO(w_col, fcs_state_get_col(soft_thread->current_pos.s, w));
-            i = strecpy(w_col + 1, (char *)(l->pile));
-            fcs_col_len(w_col) = i;
+            i = strecpy(w_col + 1, (fcs_card *)(l->pile));
+            fcs_col_len(w_col) = (fcs_card)i;
             soft_thread->current_pos.stack_hashes[w] = l->hash;
         }
     }
@@ -148,7 +146,7 @@ static inline fcs_pats_position *dequeue_position(
     but we still get lots of low priority action (instead of
     ignoring it completely). */
 
-    bool is_last = FALSE;
+    bool is_last = false;
     do
     {
         if ((--soft_thread->dequeue__qpos) < soft_thread->dequeue__minpos)
@@ -164,7 +162,7 @@ static inline fcs_pats_position *dequeue_position(
             }
             if (soft_thread->dequeue__minpos == 0)
             {
-                is_last = TRUE;
+                is_last = true;
             }
         }
     } while (soft_thread->queue_head[soft_thread->dequeue__qpos] == NULL);
@@ -245,8 +243,8 @@ fcs_pats_position *fc_solve_pats__new_position(
     pos->parent = parent;
     pos->node = node;
     pos->move = *m; /* struct copy */
-    pos->cluster = cluster;
-    pos->depth = depth;
+    pos->cluster = (unsigned short)cluster;
+    pos->depth = (short)depth;
     pos->num_childs = 0;
 
     p += sizeof(fcs_pats_position);
@@ -261,7 +259,7 @@ fcs_pats_position *fc_solve_pats__new_position(
         }
     }
 #endif
-    pos->num_cards_in_freecells = i;
+    pos->num_cards_in_freecells = (unsigned char)i;
 
     return pos;
 }
@@ -270,7 +268,6 @@ fcs_pats_position *fc_solve_pats__new_position(
 static inline bool check_for_exceeded(fcs_pats_thread *const soft_thread)
 {
     return ((soft_thread->status == FCS_PATS__NOSOL) &&
-            (soft_thread->max_num_checked_states >= 0) &&
             (soft_thread->num_checked_states >=
                 soft_thread->max_num_checked_states));
 }
@@ -374,24 +371,24 @@ static inline int solve(
 #define UP_LEVEL (soft_thread->solve_stack[DEPTH + 1])
     var_AUTO(mydir, soft_thread->curr_solve_dir);
 
-    *is_finished = FALSE;
+    *is_finished = false;
     while (DEPTH >= 0)
     {
         if (check_for_exceeded(soft_thread))
         {
             wrap_up_solve(soft_thread, mydir);
-            return TRUE;
+            return true;
         }
 
         var_S_SLOT(parent, LEVEL);
         /* If we've won already (or failed), we just go through the motions
-        but always return FALSE from any position.  This enables the cleanup
+        but always return false from any position.  This enables the cleanup
         of the move stack and eventual destruction of the position store. */
 
         if ((soft_thread->status != FCS_PATS__NOSOL) ||
             (parent->node->depth < parent->depth))
         {
-            LEVEL.q = FALSE;
+            LEVEL.q = false;
             --DEPTH;
             mydir = FC_SOLVE_PATS__DOWN;
             continue;
@@ -405,19 +402,19 @@ static inline int solve(
                 fc_solve_pats__get_moves(soft_thread, parent, &num_moves);
             if (!LEVEL.moves_start)
             {
-                LEVEL.q = FALSE;
+                LEVEL.q = false;
                 --DEPTH;
                 mydir = FC_SOLVE_PATS__DOWN;
                 continue;
             }
-            LEVEL.moves_end =
-                LEVEL.moves_start + (parent->num_childs = num_moves);
+            LEVEL.moves_end = LEVEL.moves_start +
+                              (parent->num_childs = (unsigned char)num_moves);
             LEVEL.move_ptr = LEVEL.moves_start;
-            LEVEL.q = FALSE;
+            LEVEL.q = false;
         }
         else
         {
-            num_moves = LEVEL.moves_end - LEVEL.moves_start;
+            num_moves = (int)(LEVEL.moves_end - LEVEL.moves_start);
         }
 
         // Make each move and either solve or queue the result.
@@ -425,7 +422,7 @@ static inline int solve(
         {
             if (UP_LEVEL.q)
             {
-                LEVEL.q = TRUE;
+                LEVEL.q = true;
             }
             else
             {
@@ -437,8 +434,8 @@ static inline int solve(
 
         if (LEVEL.move_ptr == LEVEL.moves_end)
         {
-            fc_solve_pats__free_array(
-                soft_thread, LEVEL.moves_start, fcs_pats__move, num_moves);
+            fc_solve_pats__free_array(soft_thread, LEVEL.moves_start,
+                fcs_pats__move, (size_t)num_moves);
             LEVEL.moves_start = NULL;
             --DEPTH;
             mydir = FC_SOLVE_PATS__DOWN;
@@ -478,7 +475,7 @@ static inline int solve(
                             FCS_PATS__SOLVE_LEVEL_GROW_BY;
                         soft_thread->solve_stack =
                             SREALLOC(soft_thread->solve_stack,
-                                soft_thread->max_solve_depth);
+                                (size_t)soft_thread->max_solve_depth);
                     }
                     UP_LEVEL.parent = LEVEL.pos;
                     UP_LEVEL.moves_start = NULL;
@@ -490,14 +487,14 @@ static inline int solve(
                     fc_solve_pats__queue_position(
                         soft_thread, LEVEL.pos, (LEVEL.move_ptr)->pri);
                     fc_solve_pats__undo_move(soft_thread, LEVEL.move_ptr);
-                    LEVEL.q = TRUE;
+                    LEVEL.q = true;
                     LEVEL.move_ptr++;
                     mydir = FC_SOLVE_PATS__UP;
                 }
             }
         }
     }
-    *is_finished = TRUE;
+    *is_finished = true;
     wrap_up_solve(soft_thread, mydir);
     return UP_LEVEL.q;
 #undef LEVEL
