@@ -10,8 +10,8 @@
 // reduce the memory consumption. See
 // ../docs/split-fully-connected-components-based-solver-planning.txt
 // in the Freecell Solver git repository.
-#ifndef FCS_DBM__VAL_IS_ANCESTOR
-#error FCS_DBM__VAL_IS_ANCESTOR must be defined
+#ifdef FCS_DBM__VAL_IS_ANCESTOR
+#error FCS_DBM__VAL_IS_ANCESTOR must not be defined
 #endif
 
 #include "dbm_solver_head.h"
@@ -223,7 +223,7 @@ static void *instance_run_solver_thread(void *const void_arg)
         if (instance->common.should_terminate == DONT_TERMINATE)
         {
             if (fcs_depth_multi_queue__extract(&(coll->depth_queue),
-                    &(thread->state_depth), (offloading_queue_item *)(&token)))
+                    &(thread->state_depth), &(token->key)))
             {
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wnull-dereference"
@@ -346,17 +346,11 @@ static void *instance_run_solver_thread(void *const void_arg)
 #endif
 
             if (instance_solver_thread_calc_derived_states(local_variant,
-                    &state,
-#ifdef FCS_DBM__VAL_IS_ANCESTOR
-                    token->ancestor,
-#else
-                    token,
-#endif
-                    &derived_list, &derived_list_recycle_bin,
-                    &derived_list_allocator, true))
+                    &state, token->key, &derived_list,
+                    &derived_list_recycle_bin, &derived_list_allocator, true))
             {
                 fcs_lock_lock(&instance->global_lock);
-                fcs_dbm__found_solution(&(instance->common), token, item);
+                fcs_dbm__found_solution(&(instance->common), token);
                 fcs_lock_unlock(&instance->global_lock);
                 break;
             }
@@ -366,16 +360,12 @@ static void *instance_run_solver_thread(void *const void_arg)
                 derived_iter = derived_iter->next)
             {
                 fcs_init_and_encode_state(delta_stater, local_variant,
-                    &(derived_iter->state), &(derived_iter->key));
+                    &(derived_iter->state),
+                    &(derived_iter->key_and_parent.key));
             }
 
             instance_check_multiple_keys(thread, instance, &(coll->cache_store),
-                &(coll->queue_meta_alloc), &derived_list, 1
-#ifndef FCS_DBM_WITHOUT_CACHES
-                ,
-                item->moves_to_key
-#endif
-            );
+                &(coll->queue_meta_alloc), &derived_list, 1);
 
             fcs_derived_state_list__recycle(
                 &derived_list_recycle_bin, &derived_list);
@@ -407,16 +397,13 @@ static inline void instance_alloc_num_moves(
     }
 }
 
-static inline void instance_check_key(
-    dbm_solver_thread *const thread, dbm_solver_instance *const instance,
-    const size_t key_depth, fcs_encoded_state_buffer *const key,
-    fcs_dbm_store_val parent, const unsigned char move GCC_UNUSED,
-    const fcs_which_moves_bitmask *const which_irreversible_moves_bitmask
-#ifndef FCS_DBM_WITHOUT_CACHES
-    ,
-    const fcs_fcc_move *moves_to_parent
-#endif
-)
+static inline void instance_check_key(dbm_solver_thread *const thread,
+    dbm_solver_instance *const instance, const size_t key_depth,
+    fcs_encoded_state_buffer *const key,
+    const fcs_encoded_state_buffer raw_parent,
+
+    const unsigned char move GCC_UNUSED,
+    const fcs_which_moves_bitmask *const which_irreversible_moves_bitmask)
 {
 #ifdef DEBUG_OUT
     fcs_state_locs_struct locs;
@@ -425,12 +412,8 @@ static inline void instance_check_key(
 #endif
     const_AUTO(coll, &(instance->coll));
     fcs_dbm_record *token;
-    if ((token = cache_store__has_key(&coll->cache_store, key, parent)))
+    if ((token = cache_store__has_key(&coll->cache_store, key, raw_parent)))
     {
-#ifndef FCS_DBM_WITHOUT_CACHES
-        cache_store__insert_key(
-            &(coll->cache_store), key, parent, moves_to_parent, move);
-#endif
 #if 0
         printf("key_depth = %zu ; curr_depth = %zu\n", key_depth, instance->curr_depth);
 #endif
@@ -439,9 +422,8 @@ static inline void instance_check_key(
             // Now insert it into the queue.
             fcs_lock_lock(&instance->global_lock);
 
-            fcs_depth_multi_queue__insert(&(coll->depth_queue),
-                thread->state_depth + 1,
-                (const offloading_queue_item *)(&token));
+            fcs_depth_multi_queue__insert(
+                &(coll->depth_queue), thread->state_depth + 1, (&token->key));
 
             ++instance->common.count_of_items_in_queue;
             ++instance->common.num_states_in_collection;
@@ -465,19 +447,17 @@ static inline void instance_check_key(
             fcs_lock_lock(&instance->fcc_exit_points_output_lock);
             // instance->storage_lock is already locked in
             // instance_check_multiple_keys and we should not lock it here.
-#ifndef FCS_DBM__VAL_IS_ANCESTOR
+#if 0
             size_t trace_num;
             fcs_encoded_state_buffer *trace;
             calc_trace(token, &trace, &trace_num);
 #endif
             {
-#ifndef FCS_DBM__VAL_IS_ANCESTOR
+#if 0
+            size_t trace_num;
                 FccEntryPointNode fcc_entry_key;
                 fcc_entry_key.kv.key.key = trace[trace_num - 1];
                 FccEntryPointNode *val_proto = RB_FIND(FccEntryPointList,
-#else
-                FccEntryPointNode *val_proto = token->ancestor;
-#endif
                 if (!val_proto)
                 {
                     goto cleanup;
@@ -514,9 +494,9 @@ static inline void instance_check_key(
                 base64_decode(moves_to_state_enc, string_len,
                     ((unsigned char *)instance->moves_to_state),
                     &(instance->moves_to_state_len));
+#endif
             }
 
-#ifdef FCS_DBM__VAL_IS_ANCESTOR
             const fcs_encoded_state_buffer outkey = token->key;
             const size_t keysize = sizeof(outkey);
             const_SLOT(moves_to_state_len, instance);
@@ -524,21 +504,7 @@ static inline void instance_check_key(
             instance_alloc_num_moves(instance, added_moves_to_output);
             unsigned char *const moves_to_state = instance->moves_to_state;
             memcpy(&moves_to_state[moves_to_state_len], &outkey, keysize);
-#else
-            const_SLOT(moves_to_state_len, instance);
-            const size_t added_moves_to_output =
-                moves_to_state_len + trace_num - 1;
-            instance_alloc_num_moves(instance, added_moves_to_output);
-            unsigned char *const moves_to_state = instance->moves_to_state;
-            ssize_t s_trace_num = (ssize_t)trace_num - 1;
-            for (ssize_t i = s_trace_num; i > 0; --i)
-            {
-                moves_to_state[(ssize_t)moves_to_state_len + s_trace_num - i] =
-                    get_move_from_parent_to_child(instance,
-                        &(thread->delta_stater), trace[i], trace[i - 1]);
-            }
 
-#endif
             const size_t new_max_enc_len =
                 ((added_moves_to_output * 4) / 3) + 20;
 
@@ -580,11 +546,10 @@ static inline void instance_check_key(
             }
 #endif
             fflush(instance->fcc_exit_points_out_fh);
+#if 0
         cleanup:
-            fcs_lock_unlock(&instance->fcc_exit_points_output_lock);
-#ifndef FCS_DBM__VAL_IS_ANCESTOR
-            free(trace);
 #endif
+            fcs_lock_unlock(&instance->fcc_exit_points_output_lock);
         }
     }
 }
@@ -735,11 +700,8 @@ int main(int argc, char *argv[])
     {
         FccEntryPointNode *const entry_point = fcs_compact_alloc_ptr(
             &(instance.fcc_entry_points_allocator), sizeof(*entry_point));
-#ifdef FCS_DBM__VAL_IS_ANCESTOR
-        entry_point->kv.key.ancestor = entry_point;
-#else
-        fcs_dbm_record_set_parent_ptr(&(entry_point->kv.key), NULL);
-#endif
+        entry_point->kv.key.parent = entry_point->kv.key.parent;
+
         char state_base64[100] = {0};
         int state_depth = -1;
         assert(2 == sscanf(instance.fingerprint_line, "%99s %d", state_base64,
@@ -763,11 +725,9 @@ int main(int argc, char *argv[])
 // These states should be traversed - not blocked, so don't put them inside the
 // store.
 #if 0
-#ifndef FCS_DBM_WITHOUT_CACHES
-        cache_store__insert_key(&(instance.cache_store), &(key_ptr->kv.key.key), &parent_state_enc, NULL, '\0');
-#else
-        token = fc_solve_dbm_store_insert_key_value(instance.coll.store, &(key_ptr->kv.key.key), NULL, true);
-#endif
+        fcs_encoded_state_buffer null_parent;
+        fcs_init_encoded_state(&null_parent);
+        token = fc_solve_dbm_store_insert_key_value(instance.coll.store, &(key_ptr->kv.key.key), &null_parent);
 
 #if 0
         entry_point->kv.val.was_consumed =
@@ -780,8 +740,7 @@ int main(int argc, char *argv[])
         RB_INSERT(FccEntryPointList, &(instance.fcc_entry_points), entry_point);
 
         {
-            const offloading_queue_item token =
-                ((const offloading_queue_item)(&(entry_point->kv.key)));
+            const offloading_queue_item token = (entry_point->kv.key.key);
             if (was_init)
             {
                 fcs_depth_multi_queue__insert(
@@ -821,6 +780,7 @@ int main(int argc, char *argv[])
     fclose(instance.fcc_exit_points_out_fh);
     rename(fcc_exit_points_out_fn_temp, fcc_exit_points_out_fn);
     fclose(fingerprint_fh);
+    trace_solution(0, NULL, NULL, NULL, NULL);
     handle_and_destroy_instance_solution(&instance, &delta);
     free(instance.moves_to_state);
     free(instance.moves_base64_encoding_buffer);
